# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore
import qtawesome as qta

from prettyqt import widgets

MODALITIES = dict(window=QtCore.Qt.WindowModal,
                  application=QtCore.Qt.ApplicationModal)


class Dialog(QtWidgets.QDialog):

    DEFAULT_SIZE = (1500, 1000)

    def __init__(self, title="", icon=None, parent=None, horizontal=False):
        super().__init__(parent=parent)
        self.resize(*self.DEFAULT_SIZE)
        self.setWindowModality(QtCore.Qt.WindowModal)
        self.setWindowTitle(title)
        self.set_icon(icon)
        self.delete_on_close()
        self.layout = widgets.BoxLayout("horizontal" if horizontal else "vertical")
        self.setLayout(self.layout)

    def delete_on_close(self):
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)

    def add_widget(self, widget):
        self.layout.addWidget(widget)

    def set_icon(self, icon):
        if icon:
            if isinstance(icon, str):
                icon = qta.icon(icon, color="lightgray")
            self.setWindowIcon(icon)

    def add_buttonbox(self):
        button_box = widgets.DialogButtonBox()
        button_box.add_buttons(["cancel", "ok"])
        button_box.accepted.connect(self.accepted)
        button_box.rejected.connect(self.reject)
        self.layout.addWidget(button_box)

    def accepted(self):
        self.close()

    def keyPressEvent(self, e):
        if e.key() == QtCore.Qt.Key_Escape:
            self.close()
        elif e.key() == QtCore.Qt.Key_F11:
            if self.isMaximized():
                self.showNormal()
            else:
                self.showMaximized()
        else:
            super().keyPressEvent(e)

    def open(self):
        self.show()
        self.exec_()


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = Dialog()
    widget.showMaximized()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import sys

from qtpy import QtWidgets

import markdown
import docutils.core


class TextBrowser(QtWidgets.QTextBrowser):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setOpenExternalLinks(True)

    # def dragEnterEvent(self, event):
    #     u = event.mimeData().urls()
    #     for url in u:
    #         file_path = os.path.abspath(url.toLocalFile())

    #         ext = file_path.split('.')[-1]
    #         if ext in ['txt', 'md', 'markdown']:
    #             event.accept()
    #         else:
    #             event.ignore()

    # def dropEvent(self, event):
    #     event.accept()
    #     self.show_markdown(self.filePath)

    def show_markdown(self, file_path):
        with open(file_path) as f:
            file_content = f.read()
        self.setHtml(markdown.markdown(file_content))

    def show_rst(self, file_path):
        with open(file_path) as f:
            html = docutils.core.publish_string(f.read(), writer_name='html')
        self.setHtml(str(html))


def main():
    qtapp = QtWidgets.QApplication(sys.argv)
    reader = TextBrowser()
    reader.show()
    qtapp.exec_()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import pathlib
from qtpy import QtWidgets, QtCore
from prettyqt import gui, widgets


H_ALIGNMENTS = dict(left=QtCore.Qt.AlignLeft,
                    right=QtCore.Qt.AlignRight,
                    center=QtCore.Qt.AlignHCenter,
                    justify=QtCore.Qt.AlignJustify)

V_ALIGNMENTS = dict(top=QtCore.Qt.AlignTop,
                    bottom=QtCore.Qt.AlignBottom,
                    center=QtCore.Qt.AlignVCenter,
                    baseline=QtCore.Qt.AlignBaseline)

TEXT_INTERACTION = dict(none=QtCore.Qt.NoTextInteraction,
                        by_mouse=QtCore.Qt.NoTextInteraction,
                        by_keyboard=QtCore.Qt.NoTextInteraction)


class Label(QtWidgets.QLabel):

    def set_alignment(self, horizontal=None, vertical=None):
        if horizontal is None and vertical is not None:
            flag = V_ALIGNMENTS.get(vertical)
        elif vertical is None and horizontal is not None:
            flag = H_ALIGNMENTS.get(horizontal)
        elif vertical is not None and horizontal is not None:
            flag = V_ALIGNMENTS.get(vertical) | H_ALIGNMENTS.get(horizontal)
        else:
            return
        self.setAlignment(flag)

    def set_image(self, path, width=300):
        self.setScaledContents(True)
        self.set_alignment(horizontal="center")
        self.setText("<html><head/><body><p>"
                     f'<img src="{path}" width="{width}"/>'
                     "</p></body></html>")

    @classmethod
    def image_from_path(cls, path: pathlib.Path, parent=None) -> "Label":
        pixmap = gui.Pixmap.from_file(path)
        label = cls(parent=parent)
        label.setPixmap(pixmap)
        label.resize(pixmap.width(), pixmap.height())
        return label


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = Label("test")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import gui, core


def return_format(color, style=""):
    """
    Return a QTextCharFormat with the given attributes.
    """
    fmt = gui.TextCharFormat()
    fmt.set_foreground_color(color)
    if "bold" in style:
        fmt.setFontWeight(gui.Font.Bold)
    if "italic" in style:
        fmt.setFontItalic(True)

    return fmt


# Syntax styles that can be shared by all languages

STYLES = {
    "keyword": return_format([200, 120, 50], "bold"),
    "operator": return_format([150, 150, 150]),
    "brace": return_format("darkGray"),
    "defclass": return_format([20, 20, 255], "bold"),
    "string": return_format([20, 110, 100]),
    "string2": return_format([30, 120, 110]),
    "comment": return_format([128, 128, 128]),
    "self": return_format([150, 85, 140], "italic"),
    "numbers": return_format([100, 150, 190]),
}

KEYWORDS = [
    "and", "assert", "break", "class", "continue", "def",
    "del", "elif", "else", "except", "exec", "finally",
    "for", "from", "global", "if", "import", "in",
    "is", "lambda", "not", "or", "pass", "print",
    "raise", "return", "try", "while", "yield",
    "None", "True", "False",
]

# Python operators
OPERATORS = [
    "=",
    # Comparison
    "==", "!=", "<", "<=", ">", ">=",
    # Arithmetic
    "\+", "-", "\*", "/", "//", "\%", "\*\*",
    # In-place
    "\+=", "-=", "\*=", "/=", "\%=",
    # Bitwise
    "\^", "\|", "\&", "\~", ">>", "<<",
]

# Python braces
BRACES = [
    "\{", "\}", "\(", "\)", "\[", "\]",
]


class PythonHighlighter(gui.SyntaxHighlighter):
    """Syntax highlighter for the Python language.
    """

    # Python keywords

    def __init__(self, document):
        super().__init__(document)

        # Multi-line strings (expression, flag, style)
        # FIXME: The triple-quotes in these two lines will mess up the
        # syntax highlighting from this point onward
        self.tri_single = (core.RegExp("'''"), 1, STYLES["string2"])
        self.tri_double = (core.RegExp('"""'), 2, STYLES["string2"])

        rules = []

        # Keyword, operator, and brace rules
        rules += [(r"\b%s\b" % w, 0, STYLES["keyword"])
                  for w in KEYWORDS]
        rules += [(r"%s" % o, 0, STYLES["operator"])
                  for o in OPERATORS]
        rules += [(r"%s" % b, 0, STYLES["brace"])
                  for b in BRACES]

        # All other rules
        rules += [
            # 'self'
            (r"\bself\b", 0, STYLES["self"]),
            # Double-quoted string, possibly containing escape sequences
            (r'"[^"\\]*(\\.[^"\\]*)*"', 0, STYLES["string"]),
            # Single-quoted string, possibly containing escape sequences
            (r"'[^'\\]*(\\.[^'\\]*)*'", 0, STYLES["string"]),
            # 'def' followed by an identifier
            (r"\bdef\b\s*(\w+)", 1, STYLES["defclass"]),
            # 'class' followed by an identifier
            (r"\bclass\b\s*(\w+)", 1, STYLES["defclass"]),
            # From '#' until a newline
            (r"#[^\n]*", 0, STYLES["comment"]),
            # Numeric literals
            (r"\b[+-]?[0-9]+[lL]?\b", 0, STYLES["numbers"]),
            (r"\b[+-]?0[xX][0-9A-Fa-f]+[lL]?\b", 0, STYLES["numbers"]),
            (r"\b[+-]?[0-9]+(?:\.[0-9]+)?(?:[eE][+-]?[0-9]+)?\b", 0, STYLES["numbers"]),
        ]

        # Build a core.RegExp for each pattern
        self.rules = [(core.RegExp(pat), index, fmt)
                      for (pat, index, fmt) in rules]

    def highlightBlock(self, text):
        """Apply syntax highlighting to the given block of text.
        """
        # Do other syntax formatting
        for expression, nth, fmt in self.rules:
            index = expression.indexIn(text)

            while index >= 0:
                # We actually want the index of the nth match
                index = expression.pos(nth)
                length = len(expression.cap(nth))
                self.setFormat(index, length, fmt)
                index = expression.indexIn(text, index + length)

        self.setCurrentBlockState(0)

        # Do multi-line strings
        in_multiline = self.match_multiline(text, *self.tri_single)
        if not in_multiline:
            self.match_multiline(text, *self.tri_double)

    def match_multiline(self, text, delimiter, in_state, style):
        """Do highlighting of multi-line strings. ``delimiter`` should be a
        ``core.RegExp`` for triple-single-quotes or triple-double-quotes, and
        ``in_state`` should be a unique integer to represent the corresponding
        state changes when inside those strings. Returns True if we're still
        inside a multi-line string when this function is finished.
        """
        # If inside triple-single quotes, start at 0
        if self.previousBlockState() == in_state:
            start = 0
            add = 0
        # Otherwise, look for the delimiter on this line
        else:
            start = delimiter.indexIn(text)
            # Move past this match
            add = delimiter.matchedLength()

        # As long as there's a delimiter match on this line...
        while start >= 0:
            # Look for the ending delimiter
            end = delimiter.indexIn(text, start + add)
            # Ending delimiter on this line?
            if end >= add:
                length = end - start + add + delimiter.matchedLength()
                self.setCurrentBlockState(0)
            # No; multi-line string
            else:
                self.setCurrentBlockState(in_state)
                length = len(text) - start + add
            # Apply formatting
            self.setFormat(start, length, style)
            # Look for the next match
            start = delimiter.indexIn(text, start + length)

        # Return True if still inside a multi-line string, False otherwise
        return self.currentBlockState() == in_state



# -*- coding: utf-8 -*-

"""syntaxhighlighters module

contains some custom syntax highlighers
"""

from .jsonhighlighter import JsonHighlighter
from .yamlhighlighter import YamlHighlighter
from .pythonhighlighter import PythonHighlighter


__all__ = ["JsonHighlighter", "YamlHighlighter", "PythonHighlighter"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets


class TableView(QtWidgets.QTableView):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def h_header(self):
        return self.horizontalHeader()


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    dlg = widgets.MainWindow()
    status_bar = TableView()
    dlg.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui, QtCore


class KeySequence(QtGui.QKeySequence):

    def __str__(self):
        return self.toString()

    def set_color(self, color):
        if isinstance(color, str):
            self.setNamedColor(color)
        else:
            self.setRgb(*color)

    @classmethod
    def to_shortcut_str(cls, key, mod):
        mods = {QtCore.Qt.ShiftModifier: QtCore.Qt.SHIFT,
                QtCore.Qt.ControlModifier: QtCore.Qt.CTRL,
                QtCore.Qt.AltModifier: QtCore.Qt.ALT,
                QtCore.Qt.MetaModifier: QtCore.Qt.META}
        for k, v in mods.items():
            if mod & k:
                key += v
        return str(cls(key))



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui
from prettyqt import gui


class StandardItemModel(QtGui.QStandardItemModel):

    def add_item(self, label):
        item = gui.StandardItem(label)
        self.appendRow(item)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore, QtGui

ELIDE_MODES = dict(left=QtCore.Qt.ElideLeft,
                   right=QtCore.Qt.ElideRight,
                   middle=QtCore.Qt.ElideMiddle,
                   none=QtCore.Qt.ElideNone)

SELECTION_MODES = dict(left_tab=QtWidgets.QTabBar.SelectLeftTab,
                       right_tab=QtWidgets.QTabBar.SelectRightTab,
                       previous_tab=QtWidgets.QTabBar.SelectPreviousTab)

SHAPES = dict(rounded_north=QtWidgets.QTabBar.RoundedNorth,
              rounded_south=QtWidgets.QTabBar.RoundedSouth,
              rounded_west=QtWidgets.QTabBar.RoundedWest,
              rounded_east=QtWidgets.QTabBar.RoundedEast,
              triangular_north=QtWidgets.QTabBar.TriangularNorth,
              triangular_south=QtWidgets.QTabBar.TriangularSouth,
              triangular_west=QtWidgets.QTabBar.TriangularWest,
              triangular_east=QtWidgets.QTabBar.TriangularEast)


class TabBar(QtWidgets.QTabBar):
    on_detach = QtCore.Signal(int, QtCore.QPoint)

    def __init__(self, parent=None):
        super().__init__(parent)

        self.setAcceptDrops(True)
        self.set_elide_mode("right")
        self.set_remove_behaviour("left_tab")
        self.mouse_cursor = QtGui.QCursor()

    #  Send the on_detach when a tab is double clicked
    def mouseDoubleClickEvent(self, event):
        event.accept()
        self.on_detach.emit(self.tabAt(event.pos()), self.mouse_cursor.pos())

    def set_icon_size(self, size: int):
        self.setIconSize(QtCore.QSize(size, size))

    def set_tab_button(self, index, widget, position="left"):
        if position == "left":
            self.setTabButton(index, QtWidgets.QTabBar.LeftSide, widget)
        else:
            self.setTabButton(index, QtWidgets.QTabBar.RightSide, widget)

    def set_remove_behaviour(self, mode):
        """sets the remove hehaviour

        What tab should be set as current when removeTab is called
        if the removed tab is also the current tab.
        Possible values: left, right, previous
        Args:
            mode: new remove behaviour
        """
        if mode not in SELECTION_MODES:
            raise ValueError("Mode not available")
        self.setSelectionBehaviorOnRemove(SELECTION_MODES[mode])

    def set_elide_mode(self, mode):
        if mode not in ELIDE_MODES:
            raise ValueError("Mode not available")
        self.setElideMode(ELIDE_MODES[mode])



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore


class Splitter(QtWidgets.QSplitter):

    def __init__(self, orientation, parent=None):
        o = QtCore.Qt.Vertical if orientation == "vertical" else QtCore.Qt.Horizontal
        super().__init__(o, parent)

    def set_sizes(self):
        self.set_sizes()

    @classmethod
    def from_widgets(cls, widgets, horizontal: bool = False, parent=None):
        orientation = QtCore.Qt.Horizontal if horizontal else QtCore.Qt.Vertical
        splitter = cls(orientation, parent=parent)
        for widget in widgets:
            splitter.addWidget(widget)
        return splitter

    def set_expanding(self):
        self.setSizePolicy(QtWidgets.QSizePolicy.Expanding,
                           QtWidgets.QSizePolicy.Expanding)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = Splitter()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

"""core module

contains QtCore-based classes
"""

# from qtpy.QtCore import Signal

from .object import Object
from .settings import Settings
from .size import Size
from .point import Point
from .timer import Timer
from .rect import Rect
from .regexp import RegExp
from .mimedata import MimeData
from .runnable import Runnable
from .modelindex import ModelIndex
from .threadpool import ThreadPool
from .filesystemmodel import FileSystemModel

from qtpy.QtCore import Slot, Signal

__all__ = ["Object",
           "Settings",
           "Size",
           "Point",
           "Timer",
           "Rect",
           "MimeData",
           "FileSystemModel",
           "Slot",
           "RegExp",
           "Runnable",
           "ModelIndex",
           "ThreadPool",
           "Signal"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui
from prettyqt import gui


class TextCharFormat(QtGui.QTextCharFormat):

    def set_foreground_color(self, color_name):
        color = gui.Color()
        color.set_color(color_name)
        self.setForeground(color)

    def set_font_weight(self, weight):
        WEIGHTS = dict(thin=QtGui.QFont.Thin,
                       light=QtGui.QFont.Light,
                       medium=QtGui.QFont.Medium,
                       bold=QtGui.QFont.Bold)
        if weight not in WEIGHTS:
            raise ValueError("Invalid font weight")
        self.setFontWeight(WEIGHTS[weight])



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore
from prettyqt import widgets
import qtawesome as qta


class ProgressDialog(QtWidgets.QProgressDialog):
    """Progress dialog

    wrapper for QtWidgets.QProgressDialog
    """

    def __init__(self, parent=None):
        super().__init__(parent)

        progress_bar = widgets.ProgressBar()
        progress_bar.setRange(0, 0)
        progress_bar.setTextVisible(False)
        self.setBar(progress_bar)

        self.set_icon("mdi.timer-sand-empty")
        self.setWindowModality(QtCore.Qt.ApplicationModal)
        self.setWindowFlags(QtCore.Qt.WindowStaysOnTopHint | QtCore.Qt.Window)
        self.set_titlebar_buttons()
        self.setCancelButton(None)
        self.cancel()

    def set_icon(self, icon):
        if icon:
            if isinstance(icon, str):
                icon = qta.icon(icon, color="lightgray")
            self.setWindowIcon(icon)

    def open(self, message=None):
        if not message:
            message = "Processing..."
        self.setLabelText(message)
        self.show()

    def set_titlebar_buttons(self,
                             minimize: bool = False,
                             maximize: bool = False,
                             close: bool = False):
        self.setWindowFlag(QtCore.Qt.WindowMinimizeButtonHint, minimize)
        self.setWindowFlag(QtCore.Qt.WindowMaximizeButtonHint, maximize)
        self.setWindowFlag(QtCore.Qt.WindowCloseButtonHint, close)


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    widget = ProgressDialog()
    widget.show()
    widget.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""


from qtpy import QtWidgets, QtCore
from prettyqt import widgets


class DockWidget(QtWidgets.QDockWidget):
    """
    Customized DockWidget class
    contains a custom TitleBar with maximise button
    """

    def __init__(self, *args, **kwargs):
        name = kwargs.pop("name", None)
        title = kwargs.pop("title", None)
        super().__init__(*args, **kwargs)
        if name:
            self.setObjectName(name)
        if title:
            self.setWindowTitle(title)
        self.setAllowedAreas(QtCore.Qt.AllDockWidgetAreas)

    def setup_title_bar(self):
        title_bar = widgets.Widget()
        layout = widgets.BoxLayout("horizontal")
        layout.set_alignment("right")
        title_bar.setLayout(layout)
        maximise_button = widgets.PushButton()
        layout.addWidget(maximise_button)
        maximise_button.set_style_icon("maximise")
        maximise_button.clicked.connect(self.maximise)
        close_button = widgets.PushButton()
        close_button.set_style_icon("close")
        layout.addWidget(close_button)
        close_button.clicked.connect(self.close)
        self.setTitleBarWidget(title_bar)

    def maximise(self):
        if not self.isFloating():
            self.setFloating(True)
        self.showMaximized()


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    win = widgets.MainWindow()
    dock_widget = DockWidget(name="aa", title="Test")
    dock_widget.setup_title_bar()
    win.addDockWidget(QtCore.Qt.DockWidgetArea(1), dock_widget)
    win.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

from prettyqt import widgets


class MenuBar(QtWidgets.QMenuBar):

    def add_action(self, action):
        self.addAction(action)


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    win = QtWidgets.QMainWindow()
    menu_bar = MenuBar()
    win.setMenuBar(menu_bar)
    win.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui, QtCore


class RegExpValidator(QtGui.QRegExpValidator):

    def set_regex(self, regex):
        re = QtCore.QRegExp(regex)
        self.setRegExp(re)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtCore
from prettyqt import core, widgets


class ButtonDelegate(widgets.ItemDelegate):

    def __init__(self, parent, role=QtCore.Qt.UserRole):
        super().__init__(parent)
        self.fn_role = role

    def createEditor(self, parent, option, index) -> widgets.PushButton:
        label = index.data()
        btn_callback = index.data(self.fn_role)
        btn = widgets.PushButton(label, parent)
        if not btn_callback:
            btn.setEnabled(False)
        else:
            btn.clicked.connect(btn_callback)
        # btn.setStyleSheet("border:1px;")
        return btn

    def setEditorData(self, editor, index):
        editor.blockSignals(True)
        # editor.setCurrentIndex(int(index.model().data(index)))
        editor.blockSignals(False)

    def setModelData(self, editor, model, index):
        pass
        # model.setData(index, editor.text())

    @core.Slot()
    def currentIndexChanged(self):
        self.commitData.emit(self.sender())



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from typing import Callable, Optional, Any

from qtpy import QtWidgets, QtCore
from prettyqt import widgets

import qtawesome as qta


class Menu(QtWidgets.QMenu):

    def __init__(self, title="", icon=None, parent=None):
        super().__init__(title, parent=parent)
        self.set_icon(icon)
        self.setToolTipsVisible(True)

    def set_icon(self, icon):
        """set the icon for the menu

        Args:
            icon: icon to use
        """
        if isinstance(icon, str):
            icon = qta.icon(icon)
        if icon:
            self.setIcon(icon)

    def _separator(self, text: str) -> QtWidgets.QWidgetAction:
        """returns a separator showing a label

        Args:
            text: Text to show on separator

        Returns:
            Separator widget
        """
        label = QtWidgets.QLabel(text)
        label.setMinimumWidth(self.minimumWidth())
        label.setStyleSheet("background:lightgrey")
        label.setAlignment(QtCore.Qt.AlignCenter)
        separator = QtWidgets.QWidgetAction(self)
        separator.setDefaultWidget(label)
        return separator

    def add_action(self,
                   label: str,
                   callback: Callable,
                   icon: Optional[Any] = None,
                   checkable: bool = False,
                   checked: bool = False,
                   shortcut: Optional[str] = None,
                   status_tip: Optional[str] = None
                   ) -> QtWidgets.QAction:
        """Add an action to the menu

        Args:
            label: Label for button
            callback: gets called when action is triggered
            icon: icon for button (default: {None})
            checkable: as checkbox button (default: {False})
            checked: if checkable, turn on by default (default: {False})
            shortcut: Shortcut for action (a) (default: {None})
            status_tip: Status tip to be shown in status bar (default: {None})

        Returns:
            Action added to menu
        """
        action = widgets.Action(label, parent=self)
        action.triggered.connect(callback)
        action.set_icon(icon)
        action.set_shortcut(shortcut)
        if checkable:
            action.setCheckable(True)
            action.setChecked(checked)
        if status_tip is not None:
            action.setStatusTip(status_tip)
        self.addAction(action)
        return action



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore

from prettyqt import widgets

BUTTONS = dict(cancel=QtWidgets.QDialogButtonBox.Cancel,
               ok=QtWidgets.QDialogButtonBox.Ok,
               save=QtWidgets.QDialogButtonBox.Save,
               open=QtWidgets.QDialogButtonBox.Open,
               close=QtWidgets.QDialogButtonBox.Close,
               discard=QtWidgets.QDialogButtonBox.Discard,
               apply=QtWidgets.QDialogButtonBox.Apply,
               reset=QtWidgets.QDialogButtonBox.Reset,
               restore_defaults=QtWidgets.QDialogButtonBox.RestoreDefaults,
               help=QtWidgets.QDialogButtonBox.Help,
               save_all=QtWidgets.QDialogButtonBox.SaveAll,
               yes=QtWidgets.QDialogButtonBox.Yes,
               yes_to_all=QtWidgets.QDialogButtonBox.YesToAll,
               no=QtWidgets.QDialogButtonBox.No,
               no_to_all=QtWidgets.QDialogButtonBox.NoToAll,
               abort=QtWidgets.QDialogButtonBox.Abort,
               retry=QtWidgets.QDialogButtonBox.Retry,
               ignore=QtWidgets.QDialogButtonBox.Ignore)


class DialogButtonBox(QtWidgets.QDialogButtonBox):

    def set_horizontal(self):
        self.setOrientation(QtCore.Qt.Horizontal)

    def set_vertical(self):
        self.setOrientation(QtCore.Qt.Vertical)

    def add_buttons(self, buttons):
        for btn in buttons:
            if btn not in BUTTONS:
                raise ValueError("button type not available")
            self.addButton(BUTTONS[btn])

    def add_accept_button(self, button):
        self.addButton(button, QtWidgets.QDialogButtonBox.AcceptRole)

    def add_reject_button(self, button):
        self.addButton(button, QtWidgets.QDialogButtonBox.RejectRole)


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = DialogButtonBox()
    buttons = list(BUTTONS.keys())
    widget.add_buttons(buttons)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import gui, widgets

import markdown


class MarkdownWindow(widgets.MainWindow):

    def __init__(self):
        super().__init__()
        self.resize(500, 500)
        self.web_view = MarkdownWidget(self)
        self.setCentralWidget(self.web_view)
        # self.web_view.loadFinished.connect(self._load_finished)
        self.create_menu()

    # def _load_finished(self):
        # frame = self.web_view.page()
        # self.web_view.page().setViewportSize(frame.contentsSize())
        # self.resize(frame.contentsSize())
        # html_data = frame.toHtml()

    def create_menu(self):
        act_exit = widgets.Action(gui.Icon('exit.png'), '&Exit', self)
        act_exit.setShortcut('Ctrl+Q')
        act_exit.setStatusTip('Exit application')
        act_exit.triggered.connect(self.close)

        act_open = widgets.Action(gui.Icon('open.png'), '&Open', self)
        act_open.setShortcut('Ctrl+O')
        act_open.setStatusTip('Open Markdown file')
        act_open.triggered.connect(self.open_new_file)

        self.statusBar()

        menubar = self.menuBar()
        menu_file = menubar.addMenu('&File')
        menu_file.addAction(act_open)
        menu_file.addAction(act_exit)

    def open_new_file(self):
        try:
            fname = widgets.FileDialog.getOpenFileName(self,
                                                       'open file',
                                                       '',
                                                       'All Text Files (*.md *.markdown *.txt *.*)',
                                                       None,
                                                       widgets.FileDialog.DontUseNativeDialog)
            self.web_view.show_markdown(fname[0])
        except UnicodeDecodeError:
            self.statusBar().showMessage('Please select only text files')
        except IOError:
            self.statusBar().showMessage('File open canceled!')


class MarkdownWidget(widgets.TextBrowser):

    # def dragEnterEvent(self, event):
    #     u = event.mimeData().urls()
    #     for url in u:
    #         file_path = os.path.abspath(url.toLocalFile())

    #         ext = file_path.split('.')[-1]
    #         if ext in ['txt', 'md', 'markdown']:
    #             event.accept()
    #         else:
    #             event.ignore()

    # def dropEvent(self, event):
    #     event.accept()
    #     self.show_markdown(self.filePath)

    def contextMenuEvent(self, event):
        pass

    def show_markdown(self, file_path):
        markdown_file = open(file_path)
        file_content = markdown_file.read()
        self.setHtml(markdown.markdown(file_content))


def main():
    app = widgets.Application.create_default_app()
    reader = MarkdownWindow()
    # reader.web_view.load(QtCore.QUrl('blank'))
    reader.show()
    app.exec_()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui


class Color(QtGui.QColor):

    def set_color(self, color):
        if isinstance(color, str):
            self.setNamedColor(color)
        else:
            self.setRgb(*color)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtGui, QtCore

COLORS = dict(transparent=QtCore.Qt.transparent)

PEN_TYPES = dict(none=QtCore.Qt.NoPen)


class Painter(QtGui.QPainter):

    def draw_image(self, point, frame_buffer):
        self.setCompositionMode(QtGui.QPainter.CompositionMode_SourceAtop)
        self.drawImage(point, frame_buffer)
        self.end()

    def use_antialiasing(self):
        self.setRenderHint(self.Antialiasing, True)

    def fill_rect(self, rect, color):
        if color not in COLORS:
            raise ValueError("Invalid value for color.")
        self.fillRect(rect, COLORS[color])

    def set_pen(self, pen_type):
        if pen_type not in PEN_TYPES:
            raise ValueError("Invalid value for pen_type.")
        self.setPen(PEN_TYPES[pen_type])



# -*- coding: utf-8 -*-

"""gui module

contains QtGui-based classes
"""

from .regexpvalidator import RegExpValidator
from .color import Color
from .font import Font
from .icon import Icon
from .pen import Pen
from .pixmap import Pixmap
from .painter import Painter
from .palette import Palette
from .standarditem import StandardItem
from .standarditemmodel import StandardItemModel
from .textcharformat import TextCharFormat
from .syntaxhighlighter import SyntaxHighlighter
from .keysequence import KeySequence


__all__ = ["RegExpValidator",
           "Color",
           "Font",
           "Icon",
           "Pen",
           "Pixmap",
           "Painter",
           "Palette",
           "StandardItem",
           "StandardItemModel",
           "TextCharFormat",
           "SyntaxHighlighter",
           "KeySequence"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore


class Slider(QtWidgets.QSlider):

    def is_horizontal(self) -> bool:
        """check if silder is horizontal

        Returns:
            True if horizontal, else False
        """
        return self.orientation() == QtCore.Qt.Horizontal

    def is_vertical(self) -> bool:
        """check if silder is vertical

        Returns:
            True if vertical, else False
        """
        return self.orientation() == QtCore.Qt.Vertical

    def set_horizontal(self):
        """set slider orientation to horizontal
        """
        self.setOrientation(QtCore.Qt.Horizontal)

    def set_vertical(self):
        """set slider orientation to vertical
        """
        self.setOrientation(QtCore.Qt.Vertical)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from prettyqt import gui, core


class JsonHighlighter(gui.SyntaxHighlighter):

    def __init__(self, parent=None):
        """ Constructor
        """
        super().__init__(parent)

        self.symbol_format = gui.TextCharFormat()
        self.symbol_format.set_foreground_color("red")
        self.symbol_format.setFontWeight(gui.Font.Bold)

        self.name_format = gui.TextCharFormat()
        self.name_format.set_foreground_color("blue")
        self.name_format.setFontWeight(gui.Font.Bold)
        self.name_format.setFontItalic(True)

        self.value_format = gui.TextCharFormat()
        self.value_format.set_foreground_color("darkgreen")

    def highlightBlock(self, text):
        """ Highlight a block of code using the rules outlined in the Constructor
        """
        expression = core.RegExp("(\\{|\\}|\\[|\\]|\\:|\\,)")
        index = expression.indexIn(text)
        while index >= 0:
            length = expression.matchedLength()
            self.setFormat(index, length, self.symbol_format)
            index = expression.indexIn(text, index + length)

        text.replace("\\\"", "  ")

        expression = core.RegExp("\".*\" *\\:")
        expression.setMinimal(True)
        index = expression.indexIn(text)
        while index >= 0:
            length = expression.matchedLength()
            self.setFormat(index, length - 1, self.name_format)
            index = expression.indexIn(text, index + length)

        expression = core.RegExp("\\: *\".*\"")
        expression.setMinimal(True)
        index = expression.indexIn(text)
        while index >= 0:
            length = expression.matchedLength()
            self.setFormat(index, length - 1, self.value_format)
            index = expression.indexIn(text, index + length)



# -*- coding: utf-8 -*-

"""widgets module

contains QtWidgets-based classes
"""

from .application import Application

from .widget import Widget
from .graphicsitem import GraphicsItem
from .style import Style
from .styleoption import StyleOption
from .dialog import Dialog
from .messagebox import MessageBox

from .formlayout import FormLayout
from .boxlayout import BoxLayout
from .gridlayout import GridLayout

from .slider import Slider
from .styleoptionslider import StyleOptionSlider
from .frame import Frame

from .action import Action
from .toolbutton import ToolButton
from .menu import Menu
from .menubar import MenuBar
from .statusbar import StatusBar
from .tabbar import TabBar
from .tabwidget import TabWidget
from .toolbar import Toolbar
from .headerview import HeaderView
from .dockwidget import DockWidget
from .label import Label
from .pushbutton import PushButton
from .radiobutton import RadioButton
from .combobox import ComboBox
from .checkbox import CheckBox
from .lineedit import LineEdit
from .textedit import TextEdit
from .plaintextedit import PlainTextEdit
from .textbrowser import TextBrowser
from .progressbar import ProgressBar
from .treeview import TreeView
from .tableview import TableView

from .splashscreen import SplashScreen
from .progressdialog import ProgressDialog
from .filedialog import FileDialog
from .dialogbuttonbox import DialogButtonBox
from .groupbox import GroupBox
from .splitter import Splitter

from .mainwindow import MainWindow

from .itemdelegate import ItemDelegate

from .composed.spanslider import SpanSlider
from .composed.waitingspinner import WaitingSpinner
from .composed.callout import Callout
from .composed.markdownwidget import MarkdownWindow, MarkdownWidget
from .composed.imageviewer import ImageViewer
from .composed.popupinfo import PopupInfo
from .composed.buttondelegate import ButtonDelegate

__all__ = ["Application",
           "Widget",
           "GraphicsItem",
           "Style",
           "StyleOption",
           "Dialog",
           "MessageBox",
           "FormLayout",
           "BoxLayout",
           "GridLayout",
           "Slider",
           "StyleOptionSlider",
           "Frame",
           "Action",
           "ToolButton",
           "Menu",
           "MenuBar",
           "StatusBar",
           "TabWidget",
           "TabBar",
           "Toolbar",
           "HeaderView",
           "DockWidget",
           "Label",
           "PushButton",
           "RadioButton",
           "ComboBox",
           "CheckBox",
           "LineEdit",
           "TextEdit",
           "PlainTextEdit",
           "TextBrowser",
           "ProgressBar",
           "TreeView",
           "TableView",
           "SplashScreen",
           "ProgressDialog",
           "FileDialog",
           "DialogButtonBox",
           "GroupBox",
           "Splitter",
           "MainWindow",
           "ItemDelegate",
           "SpanSlider",
           "WaitingSpinner",
           "Callout",
           "PopupInfo",
           "ButtonDelegate",
           "Image",
           "ImageViewer",
           "MarkdownWindow",
           "MarkdownWidget"]



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore
from prettyqt import widgets


MODES = dict(maximum=QtWidgets.QLayout.SetMaximumSize,
             fixed=QtWidgets.QLayout.SetFixedSize)

ALIGNMENTS = dict(left=QtCore.Qt.AlignLeft,
                  right=QtCore.Qt.AlignRight,
                  top=QtCore.Qt.AlignTop,
                  bottom=QtCore.Qt.AlignBottom)


class BoxLayout(QtWidgets.QBoxLayout):

    def __init__(self, orientation, parent=None):
        o = self.TopToBottom if orientation == "vertical" else self.LeftToRight
        super().__init__(o, parent)

    def __getitem__(self, index):
        return self.itemAt(index)

    def set_size_mode(self, mode: str):
        if mode not in MODES:
            raise ValueError(f"{mode} not a valid size mode.")
        self.setSizeConstraint(MODES[mode])

    def set_alignment(self, alignment):
        if alignment not in ALIGNMENTS:
            raise ValueError(f"{alignment} not a valid alignment.")
        self.setAlignment(ALIGNMENTS[alignment])


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    layout = BoxLayout("vertical")
    widget = widgets.Widget()
    widget.setLayout(layout)
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets
from prettyqt import widgets


class StatusBar(QtWidgets.QStatusBar):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.progress_bar = widgets.ProgressBar()

    def setup_default_bar(self):
        # This is simply to show the bar
        self.progress_bar.hide()
        self.progress_bar.setRange(0, 0)
        self.progress_bar.setFixedSize(200, 20)
        self.progress_bar.setTextVisible(False)
        self.addPermanentWidget(self.progress_bar)

    def add_action(self, action):
        self.addAction(action)


if __name__ == "__main__":
    import sys
    app = widgets.Application(sys.argv)
    dlg = widgets.MainWindow()
    status_bar = StatusBar()
    status_bar.setup_default_bar()
    dlg.setStatusBar(status_bar)
    dlg.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

import qtawesome as qta
from qtpy import QtWidgets

from prettyqt import widgets


class PushButton(QtWidgets.QPushButton):

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def set_icon(self, icon):
        if isinstance(icon, str):
            icon = qta.icon(icon)
        if icon:
            self.setIcon(icon)

    def set_style_icon(self, icon):
        STYLES = dict(close=QtWidgets.QStyle.SP_TitleBarCloseButton,
                      maximise=QtWidgets.QStyle.SP_TitleBarMaxButton)
        qicon = self.style().standardIcon(STYLES[icon], None, self)
        self.setIcon(qicon)


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = PushButton("This is a test")
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore
import qtawesome as qta

from prettyqt import widgets


class MessageBox(QtWidgets.QMessageBox):

    def set_horizontal(self):
        self.setOrientation(QtCore.Qt.Horizontal)

    def set_vertical(self):
        self.setOrientation(QtCore.Qt.Vertical)

    def set_icon(self, icon):
        if icon:
            if isinstance(icon, str):
                icon = qta.icon(icon, color="lightgray")
            self.setWindowIcon(icon)


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = MessageBox()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets

SELECTION_MODES = dict(single=QtWidgets.QAbstractItemView.SingleSelection,
                       extended=QtWidgets.QAbstractItemView.ExtendedSelection,
                       multi=QtWidgets.QAbstractItemView.MultiSelection,
                       none=QtWidgets.QAbstractItemView.NoSelection)


class TreeView(QtWidgets.QTreeView):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # visual settings
        self.setAnimated(True)
        self.setRootIsDecorated(False)
        self.setAllColumnsShowFocus(True)

        # misc
        self.setUniformRowHeights(True)
        self.set_selection_mode("extended")

    def h_header(self):
        return self.header()

    def set_selection_mode(self, mode: str):
        """set selection mode for given item view

        Allowed values are "single", "extended", "multi" or "none"

        Args:
            mode: selection mode to use

        Raises:
            ValueError: mode does not exist
        """
        if mode not in SELECTION_MODES:
            raise ValueError("Format must be either 'single', 'extended',"
                             "'multi' or 'None'")
        self.setSelectionMode(SELECTION_MODES[mode])


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    dlg = QtWidgets.QMainWindow()
    status_bar = TreeView()
    dlg.show()
    app.exec_()



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets
import qtawesome as qta


class Action(QtWidgets.QAction):

    def set_enabled(self):
        self.setEnabled(True)

    def set_disabled(self):
        self.setEnabled(False)

    def set_tooltip(self, text):
        self.setToolTip(text)

    def set_icon(self, icon):
        if isinstance(icon, str):
            icon = qta.icon(icon)
        if icon:
            self.setIcon(icon)

    def set_shortcut(self, shortcut):
        if shortcut:
            self.setShortcut(shortcut)



# -*- coding: utf-8 -*-
"""
@author: Philipp Temminghoff
"""

from qtpy import QtWidgets, QtCore

from prettyqt import widgets

H_ALIGNMENTS = dict(left=QtCore.Qt.AlignLeft,
                    right=QtCore.Qt.AlignRight,
                    center=QtCore.Qt.AlignHCenter)


class GroupBox(QtWidgets.QGroupBox):

    def set_alignment(self):
        self.setAlignment(QtCore.Qt.Vertical)


if __name__ == "__main__":
    app = widgets.Application.create_default_app()
    widget = GroupBox()
    widget.show()
    app.exec_()



# -*- coding: utf-8 -*-

"""Top-level package for PrettyQt."""

from . import core
from . import widgets
from . import gui
from . import syntaxhighlighters

__author__ = """Philipp Temminghoff"""
__email__ = 'phil65@kodi.tv'
__version__ = '0.13.1'

__all__ = ["core", "widgets", "gui", "syntaxhighlighters"]
